#!/bin/bash


export DG84_VERSION="redhat-datagrid-8.4.0.CD20220609-server.zip"
export script_dir="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
DG_831_URL=http://download.eng.rdu2.redhat.com/released/JBoss-middleware/datagrid/8.3.1/redhat-datagrid-8.3.1-server.zip
DG_84_URL=http://download.eng.bos.redhat.com/rcm-guest/staging/jboss-dg/JDG-8.4.x-CD/JDG-8.4.0.CD20220609/redhat-datagrid-8.4.0.CD20220609-server.zip

function downloadDGSource() {
  if [[ ! -e "redhat-datagrid-8.3.1-server.zip" ]]; then
      wget ${DG_831_URL}
  fi
  rm -rf redhat-datagrid-8.3.1-server
  unzip redhat-datagrid-8.3.1-server.zip
}

function downloadDGTarget() {
  if [[ ! -e ${DG84_VERSION} ]]; then
      wget ${DG_84_URL}
  fi
  rm -rf redhat-datagrid-8.4.0-server
  unzip ${DG84_VERSION}
}

function is-ready() {
   local offset=$1
   $(curl --digest -u user:passwd-123 --silent --output /dev/null http://localhost:$(( 11222 + offset ))/rest/v2/cache-managers/default/health)
}

function startDGSource() {
  mkdir -p logs
  nohup redhat-datagrid-8.3.1-server/bin/server.sh -Djgroups.mcast_addr=234.99.54.14 -Djgroups.mcast_port=55555 > logs/server-source.log &
  while ! is-ready 0 >/dev/null
  do
   echo "waiting for server to start"
   sleep 5;
  done
  echo "SOURCE SERVER STARTED"
}

function startDGTarget() {
  nohup redhat-datagrid-8.4.0-server/bin/server.sh -o 100 -Djgroups.mcast_addr=234.99.54.15 > logs/server-target.log &
  while ! is-ready 100 >/dev/null
  do
   echo "waiting for server to start"
   sleep 5;
  done
  echo "TARGET SERVER STARTED"
}

function createUser() {
  local USER="user"
  local PASS="passwd-123"
  redhat-datagrid-8.3.1-server/bin/cli.sh user create $USER -p $PASS -g admin &>/dev/null
  redhat-datagrid-8.4.0-server/bin/cli.sh user create $USER -p $PASS -g admin &>/dev/null
}

function createRemoteStore() {
  curl --digest -u user:passwd-123 -X POST --output - http://127.0.0.1:11322/rest/v2/caches/eviction -H "Content-Type: application/json" --data-binary "@${script_dir}/assets/eviction-remote-security.json"
}

function createSourceCache() {
  curl --digest -u user:passwd-123 -X POST --output - http://127.0.0.1:11222/rest/v2/caches/eviction -H "Content-Type: application/json" --data-binary "@${script_dir}/assets/eviction.json"
}

function insertSomeEntriesIntoEvictionSource() {
  "${script_dir}"/jbang --fresh "${script_dir}"/Load.java --entries 5000 --write-batch 1000 --phrase-size 100 --hotrodversion 2.8 --version 8.4 --cache-name eviction
}

function doRollingUpgrade() {
  curl --digest -u user:passwd-123 -X POST http://127.0.0.1:11322/rest/v2/caches/eviction?action=sync-data
}

function disconnectFromSourceCluster() {
  local CACHE_NAME='eviction'
  curl --digest -u user:passwd-123 -X POST http://127.0.0.1:11322/rest/v2/caches/"$CACHE_NAME"?action=disconnect-source
#  curl --digest -u user:passwd-123 -X$ACTION http://127.0.0.1:14222/rest/v2/caches/"$CACHE_NAME"?action=disconnect-source
}

#It's failing here, it's not showing 5000 entries
function checkMigrateData() {
  echo -e "\nCHECKING MIGRATED DATA"
  local CACHE_NAME='eviction'
  response=$(curl --silent --digest -u user:passwd-123 http://127.0.0.1:11322/rest/v2/caches/${CACHE_NAME}?action=size)
   if [ "$response" -eq "5000" ]; then
     echo "${CACHE_NAME} cache - TEST PASSED"
     echo "${response} entries"
   else
     echo "${CACHE_NAME} cache - TEST FAILED"
     echo "${response} entries IT SHOULD BE 5000"
   fi
}

function cleanupResources() {
  curl -X DELETE --digest -u user:passwd-123 http://127.0.0.1:11322/rest/v2/caches/eviction
  curl -X DELETE --digest -u user:passwd-123 http://127.0.0.1:11222/rest/v2/caches/eviction
}
